@@ -99,6 +99,7 @@ import org.apache.calcite.sql.parser.SqlParser;
 import org.apache.calcite.sql.parser.SqlParserImplFactory;
 import org.apache.calcite.sql.parser.impl.SqlParserImpl;
 import org.apache.calcite.sql.type.ExtraSqlTypes;
+import org.apache.calcite.sql.type.MeasureSqlType;
 import org.apache.calcite.sql.type.SqlTypeName;
 import org.apache.calcite.sql.util.SqlOperatorTables;
 import org.apache.calcite.sql.validate.SqlConformance;
@@ -130,6 +131,7 @@ import java.util.List;
 import java.util.Map;
 import java.util.Set;
 import java.util.function.Supplier;
+import java.util.function.UnaryOperator;
 
 import static org.apache.calcite.linq4j.Nullness.castNonNull;
 import static org.apache.calcite.util.Static.RESOURCE;
@@ -175,21 +177,22 @@ public class CalcitePrepareImpl implements CalcitePrepare {
 
   @Override public ParseResult parse(
       Context context, String sql) {
-    return parse_(context, sql, false, false, false);
+    return parse_(context, sql, false, false, false, false);
   }
 
   @Override public ConvertResult convert(Context context, String sql) {
-    return (ConvertResult) parse_(context, sql, true, false, false);
+    return (ConvertResult) parse_(context, sql, true, false, false, false);
   }
 
-  @Override public AnalyzeViewResult analyzeView(Context context, String sql, boolean fail) {
-    return (AnalyzeViewResult) parse_(context, sql, true, true, fail);
+  @Override public AnalyzeViewResult analyzeView(Context context, String sql,
+      boolean fail) {
+    return (AnalyzeViewResult) parse_(context, sql, true, true, fail, true);
   }
 
   /** Shared implementation for {@link #parse}, {@link #convert} and
    * {@link #analyzeView}. */
   private ParseResult parse_(Context context, String sql, boolean convert,
-      boolean analyze, boolean fail) {
+      boolean analyze, boolean fail, boolean embeddedQuery) {
     final JavaTypeFactory typeFactory = context.getTypeFactory();
     CalciteCatalogReader catalogReader =
         new CalciteCatalogReader(
@@ -204,7 +207,9 @@ public class CalcitePrepareImpl implements CalcitePrepare {
     } catch (SqlParseException e) {
       throw new RuntimeException("parse failed", e);
     }
-    final SqlValidator validator = createSqlValidator(context, catalogReader);
+    final SqlValidator validator =
+        createSqlValidator(context, catalogReader,
+            c -> c.withEmbeddedQuery(embeddedQuery));
     SqlNode sqlNode1 = validator.validate(sqlNode);
     if (convert) {
       return convert_(
@@ -336,7 +341,7 @@ public class CalcitePrepareImpl implements CalcitePrepare {
     for (RelDataTypeField field : targetRowType.getFieldList()) {
       final int x = columnMapping.indexOf(field.getIndex());
       if (x >= 0) {
-        assert Util.skip(columnMapping, x + 1).indexOf(field.getIndex()) < 0
+        assert !Util.skip(columnMapping, x + 1).contains(field.getIndex())
             : "column projected more than once; should have checked above";
         continue; // target column is projected
       }
@@ -660,7 +665,9 @@ public class CalcitePrepareImpl implements CalcitePrepare {
             Meta.StatementType.OTHER_DDL);
       }
 
-      final SqlValidator validator = preparingStmt.createSqlValidator(catalogReader);
+      final SqlValidator validator =
+          preparingStmt.createSqlValidator(catalogReader,
+              UnaryOperator.identity());
 
       preparedResult =
           preparingStmt.prepareSql(sqlNode, Object.class, validator, true);
@@ -682,7 +689,7 @@ public class CalcitePrepareImpl implements CalcitePrepare {
           preparingStmt.prepareQueryable(query.queryable, x);
       statementType = getStatementType(preparedResult);
     } else {
-      assert query.rel != null;
+      requireNonNull(query.rel);
       x = query.rel.getRowType();
       preparedResult = preparingStmt.prepareRel(query.rel);
       statementType = getStatementType(preparedResult);
@@ -734,7 +741,8 @@ public class CalcitePrepareImpl implements CalcitePrepare {
   }
 
   private static SqlValidator createSqlValidator(Context context,
-      CalciteCatalogReader catalogReader) {
+      CalciteCatalogReader catalogReader,
+      UnaryOperator<SqlValidator.Config> configTransform) {
     final SqlOperatorTable opTab0 =
         context.config().fun(SqlOperatorTable.class,
             SqlStdOperatorTable.instance());
@@ -744,11 +752,13 @@ public class CalcitePrepareImpl implements CalcitePrepare {
     final SqlOperatorTable opTab = SqlOperatorTables.chain(list);
     final JavaTypeFactory typeFactory = context.getTypeFactory();
     final CalciteConnectionConfig connectionConfig = context.config();
-    final SqlValidator.Config config = SqlValidator.Config.DEFAULT
-        .withLenientOperatorLookup(connectionConfig.lenientOperatorLookup())
-        .withConformance(connectionConfig.conformance())
-        .withDefaultNullCollation(connectionConfig.defaultNullCollation())
-        .withIdentifierExpansion(true);
+    final SqlValidator.Config config =
+        configTransform.apply(
+            SqlValidator.Config.DEFAULT
+                .withLenientOperatorLookup(connectionConfig.lenientOperatorLookup())
+                .withConformance(connectionConfig.conformance())
+                .withDefaultNullCollation(connectionConfig.defaultNullCollation())
+                .withIdentifierExpansion(true));
     return new CalciteSqlValidator(opTab, catalogReader, typeFactory,
         config);
   }
@@ -801,13 +811,19 @@ public class CalcitePrepareImpl implements CalcitePrepare {
 
   private static ColumnMetaData.AvaticaType avaticaType(JavaTypeFactory typeFactory,
       RelDataType type, @Nullable RelDataType fieldType) {
-    final String typeName = getTypeName(type);
+    final String typeName;
+    if (type instanceof MeasureSqlType) {
+      type = requireNonNull(type.getMeasureElementType(), "measure type");
+      typeName = "MEASURE<" + getTypeName(type) + ">";
+    } else {
+      typeName = getTypeName(type);
+    }
     if (type.getComponentType() != null) {
       final ColumnMetaData.AvaticaType componentType =
           avaticaType(typeFactory, type.getComponentType(), null);
       final Type clazz = typeFactory.getJavaClass(type.getComponentType());
-      final ColumnMetaData.Rep rep = ColumnMetaData.Rep.of(clazz);
-      assert rep != null;
+      final ColumnMetaData.Rep rep =
+          requireNonNull(ColumnMetaData.Rep.of(clazz));
       return ColumnMetaData.array(componentType, typeName, rep);
     } else {
       int typeOrdinal = getTypeOrdinal(type);
@@ -826,8 +842,8 @@ public class CalcitePrepareImpl implements CalcitePrepare {
       default:
         final Type clazz =
             typeFactory.getJavaClass(Util.first(fieldType, type));
-        final ColumnMetaData.Rep rep = ColumnMetaData.Rep.of(clazz);
-        assert rep != null;
+        final ColumnMetaData.Rep rep =
+            requireNonNull(ColumnMetaData.Rep.of(clazz));
         return ColumnMetaData.scalar(typeOrdinal, typeName, rep);
       }
     }
@@ -1108,7 +1124,8 @@ public class CalcitePrepareImpl implements CalcitePrepare {
       // View may have different schema path than current connection.
       final CatalogReader catalogReader =
           this.catalogReader.withSchemaPath(schemaPath);
-      SqlValidator validator = createSqlValidator(catalogReader);
+      SqlValidator validator =
+          createSqlValidator(catalogReader, c -> c.withEmbeddedQuery(true));
       final SqlToRelConverter.Config config =
           SqlToRelConverter.config().withTrimUnusedFields(true);
       SqlToRelConverter sqlToRelConverter =
@@ -1120,14 +1137,16 @@ public class CalcitePrepareImpl implements CalcitePrepare {
       return root;
     }
 
-    protected SqlValidator createSqlValidator(CatalogReader catalogReader) {
+    protected SqlValidator createSqlValidator(CatalogReader catalogReader,
+        UnaryOperator<SqlValidator.Config> configTransform) {
       return CalcitePrepareImpl.createSqlValidator(context,
-          (CalciteCatalogReader) catalogReader);
+          (CalciteCatalogReader) catalogReader, configTransform);
     }
 
     @Override protected SqlValidator getSqlValidator() {
       if (sqlValidator == null) {
-        sqlValidator = createSqlValidator(catalogReader);
+        sqlValidator =
+            createSqlValidator(catalogReader, UnaryOperator.identity());
       }
       return sqlValidator;
     }
