@@ -103,6 +103,16 @@ public enum SqlConformanceEnum implements SqlConformance {
     }
   }
 
+  @Override public boolean isSupportedDualTable() {
+    switch (this) {
+    case MYSQL_5:
+    case ORACLE_10:
+    case ORACLE_12:
+      return true;
+    default:
+      return false;
+    }
+  }
 
   @Override public boolean isGroupByAlias() {
     switch (this) {
@@ -290,6 +300,19 @@ public enum SqlConformanceEnum implements SqlConformance {
     }
   }
 
+  @Override public boolean allowNiladicConstantWithoutParentheses() {
+    switch (this) {
+    case ORACLE_10:
+    case ORACLE_12:
+    case DEFAULT:
+    case LENIENT:
+    case BABEL:
+      return true;
+    default:
+      return false;
+    }
+  }
+
   @Override public boolean allowExplicitRowValueConstructor() {
     switch (this) {
     case DEFAULT:
@@ -445,4 +468,33 @@ public enum SqlConformanceEnum implements SqlConformance {
       return false;
     }
   }
+
+  @Override public boolean checkedArithmetic() {
+    switch (this) {
+    case DEFAULT:
+    case LENIENT:
+    case BABEL:
+    case ORACLE_10:
+    case ORACLE_12:
+    case PRAGMATIC_99:
+    case PRESTO:
+    case PRAGMATIC_2003:
+    case STRICT_92:
+    case STRICT_99:
+    case STRICT_2003:
+    case MYSQL_5:
+      // MySQL in strict mode uses checked arithmetic
+      // https://dev.mysql.com/doc/refman/8.4/en/sql-mode.html#sql-mode-strict
+    default:
+      return false;
+      // Postgres also has checked arithmetic, but there is no such conformance
+    case BIG_QUERY:
+      // BigQuery throws on overflow
+      // https://cloud.google.com/bigquery/docs/reference/standard-sql/operators
+    case SQL_SERVER_2008:
+      // SET ARITHABORT OFF can be used to turn off overflow behavior, but the default is ON
+      // https://learn.microsoft.com/en-us/sql/t-sql/statements/set-arithabort-transact-sql
+      return true;
+    }
+  }
 }
