@@ -21,6 +21,8 @@ import org.apache.calcite.util.TestUnsafe;
 import org.apache.calcite.util.Unsafe;
 import org.apache.calcite.util.Util;
 
+import org.checkerframework.checker.nullness.qual.Nullable;
+
 import java.io.BufferedReader;
 import java.io.BufferedWriter;
 import java.io.File;
@@ -52,6 +54,10 @@ import java.util.TreeSet;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 
+import static java.lang.Integer.parseInt;
+import static java.lang.Long.parseLong;
+import static java.util.Objects.requireNonNull;
+
 /**
  * ConcurrentTestCommandScript creates instances of
  * {@link ConcurrentTestCommand} that perform specific actions in a specific
@@ -111,7 +117,7 @@ public class ConcurrentTestCommandScript
   private static final String PLUGIN = "@plugin";
 
   private static final String SQL = "";
-  private static final String EOF = null;
+  private static final @Nullable String EOF = null;
 
   private static final StateAction[] STATE_TABLE = {
       new StateAction(
@@ -253,7 +259,7 @@ public class ConcurrentTestCommandScript
   private final Map<Integer, ResultsReader> threadResultsReaders =
       new HashMap<>();
 
-  public ConcurrentTestCommandScript() throws IOException {
+  public ConcurrentTestCommandScript() {
     super();
   }
 
@@ -276,7 +282,7 @@ public class ConcurrentTestCommandScript
    * Gets ready to execute: loads script FILENAME applying external variable
    * BINDINGS.
    */
-  private void prepare(String filename, List<String> bindings)
+  private void prepare(String filename, @Nullable List<String> bindings)
       throws IOException {
     vars = new VariableTable();
     CommandParser parser = new CommandParser();
@@ -349,7 +355,7 @@ public class ConcurrentTestCommandScript
 
   protected void executeCommands(int threadId, List<String> commands)
       throws Exception {
-    if (commands == null || commands.size() == 0) {
+    if (commands.isEmpty()) {
       return;
     }
 
@@ -398,8 +404,7 @@ public class ConcurrentTestCommandScript
           try (Statement stmt = connection.createStatement()) {
             int rows = stmt.executeUpdate(sql);
             if (rows != 1) {
-              storeMessage(
-                  threadId, String.valueOf(rows) + " rows affected.");
+              storeMessage(threadId, rows + " rows affected.");
             } else {
               storeMessage(threadId, "1 row affected.");
             }
@@ -537,8 +542,8 @@ public class ConcurrentTestCommandScript
     printThreadResults(out, results.get(CLEANUP_THREAD_ID));
   }
 
-  private void printThreadResults(PrintWriter out, String[] threadResult)
-      throws IOException {
+  private void printThreadResults(PrintWriter out,
+      String @Nullable[] threadResult) {
     if (threadResult == null) {
       return;
     }
@@ -660,10 +665,10 @@ public class ConcurrentTestCommandScript
 
   /** State datum. */
   private static class StateDatum {
-    final String x;
+    final @Nullable String x;
     final String y;
 
-    StateDatum(String x, String y) {
+    StateDatum(@Nullable String x, String y) {
       this.x = x;
       this.y = y;
     }
@@ -671,12 +676,12 @@ public class ConcurrentTestCommandScript
 
 
   /** Symbol table of script variables. */
-  private class VariableTable {
+  private static class VariableTable {
     private final Map<String, String> map;
 
     // matches $$, $var, ${var}
     private final Pattern symbolPattern =
-        Pattern.compile("\\$((\\$)|([A-Za-z]\\w*)|\\{([A-Za-z]\\w*)\\})");
+        Pattern.compile("\\$((\\$)|([A-Za-z]\\w*)|\\{([A-Za-z]\\w*)})");
 
     VariableTable() {
       map = new HashMap<>();
@@ -707,7 +712,7 @@ public class ConcurrentTestCommandScript
     }
 
     // returns null is SYM is not defined
-    public String get(String sym) {
+    public @Nullable String get(String sym) {
       if (isDefined(sym)) {
         return map.get(sym);
       } else {
@@ -811,7 +816,7 @@ public class ConcurrentTestCommandScript
 
     // Parses a set of VAR=VAL pairs from the command line, and saves it for
     // later application.
-    public void rememberVariableRebindings(List<String> pairs) {
+    public void rememberVariableRebindings(@Nullable List<String> pairs) {
       if (pairs == null) {
         return;
       }
@@ -828,7 +833,7 @@ public class ConcurrentTestCommandScript
     }
 
     // trace loading of a script
-    private void trace(String prefix, Object message) {
+    private void trace(@Nullable String prefix, Object message) {
       if (verbose && !quiet) {
         if (prefix != null) {
           System.out.print(prefix + ": ");
@@ -854,7 +859,7 @@ public class ConcurrentTestCommandScript
           Map<String, String> commandStateMap = lookupState(state);
           final String command;
           boolean isSql = false;
-          if (line.equals("") || line.startsWith("--")) {
+          if (line.isEmpty() || line.startsWith("--")) {
             continue;
           } else if (line.startsWith("@")) {
             command = firstWord(line);
@@ -877,7 +882,7 @@ public class ConcurrentTestCommandScript
           }
           if (changeState) {
             String nextState = commandStateMap.get(command);
-            assert nextState != null;
+            requireNonNull(nextState, "nextState");
             if (!nextState.equals(state)) {
               doEndOfState(state);
             }
@@ -898,14 +903,17 @@ public class ConcurrentTestCommandScript
     }
 
     private void loadSql(String sql) {
-      if (SETUP_STATE.equals(state)) {
+      switch (state) {
+      case SETUP_STATE:
         trace("@setup", sql);
         setupCommands.add(sql);
-      } else if (CLEANUP_STATE.equals(state)) {
+        break;
+      case CLEANUP_STATE:
         trace("@cleanup", sql);
         cleanupCommands.add(sql);
-      } else if (
-          THREAD_STATE.equals(state) || REPEAT_STATE.equals(state)) {
+        break;
+      case THREAD_STATE:
+      case REPEAT_STATE:
         boolean isSelect = isSelect(sql);
         trace(sql);
         for (int i = threadId; i < nextThreadId; i++) {
@@ -914,8 +922,9 @@ public class ConcurrentTestCommandScript
           addCommand(i, order, cmd);
         }
         order++;
-      } else {
-        assert false;
+        break;
+      default:
+        throw new AssertionError();
       }
     }
 
@@ -978,19 +987,23 @@ public class ConcurrentTestCommandScript
 
       } else if (REPEAT.equals(command)) {
         String arg = line.substring(REPEAT_LEN).trim();
-        repeatCount = Integer.parseInt(vars.expand(arg));
+        repeatCount = parseInt(vars.expand(arg));
         trace("start @repeat block", repeatCount);
         assert repeatCount > 0 : "Repeat count must be > 0";
         in.mark(REPEAT_READ_AHEAD_LIMIT);
 
       } else if (END.equals(command)) {
-        if (SETUP_STATE.equals(state)) {
+        switch (state) {
+        case SETUP_STATE:
           trace("end @setup");
-        } else if (CLEANUP_STATE.equals(state)) {
+          break;
+        case CLEANUP_STATE:
           trace("end @cleanup");
-        } else if (THREAD_STATE.equals(state)) {
+          break;
+        case THREAD_STATE:
           threadId = nextThreadId;
-        } else if (REPEAT_STATE.equals(state)) {
+          break;
+        case REPEAT_STATE:
           trace("repeating");
           repeatCount--;
           if (repeatCount > 0) {
@@ -999,15 +1012,16 @@ public class ConcurrentTestCommandScript
             } catch (IOException e) {
               throw new IllegalStateException(
                   "Unable to reset reader -- repeat "
-                  + "contents must be less than "
-                  + REPEAT_READ_AHEAD_LIMIT + " bytes");
+                      + "contents must be less than "
+                      + REPEAT_READ_AHEAD_LIMIT + " bytes");
             }
 
             trace("end @repeat block");
             return false;   // don't change the state
           }
-        } else {
-          assert false;
+          break;
+        default:
+          throw new AssertionError();
         }
 
       } else if (SYNC.equals(command)) {
@@ -1020,7 +1034,7 @@ public class ConcurrentTestCommandScript
       } else if (TIMEOUT.equals(command)) {
         String args = line.substring(TIMEOUT_LEN).trim();
         String millisStr = vars.expand(firstWord(args));
-        long millis = Long.parseLong(millisStr);
+        long millis = parseLong(millisStr);
         assert millis >= 0L : "Timeout must be >= 0";
 
         String sql = readSql(skipFirstWord(args).trim(), in);
@@ -1037,7 +1051,7 @@ public class ConcurrentTestCommandScript
       } else if (ROWLIMIT.equals(command)) {
         String args = line.substring(ROWLIMIT_LEN).trim();
         String limitStr = vars.expand(firstWord(args));
-        int limit = Integer.parseInt(limitStr);
+        int limit = parseInt(limitStr);
         assert limit >= 0 : "Rowlimit must be >= 0";
 
         String sql = readSql(skipFirstWord(args).trim(), in);
@@ -1135,8 +1149,8 @@ public class ConcurrentTestCommandScript
         String arg = vars.expand(line.substring(FETCH_LEN).trim());
         trace("@fetch", arg);
         long millis = 0L;
-        if (arg.length() > 0) {
-          millis = Long.parseLong(arg);
+        if (!arg.isEmpty()) {
+          millis = parseLong(arg);
           assert millis >= 0L : "Fetch timeout must be >= 0";
         }
 
@@ -1158,7 +1172,7 @@ public class ConcurrentTestCommandScript
       } else if (SLEEP.equals(command)) {
         String arg = vars.expand(line.substring(SLEEP_LEN).trim());
         trace("@sleep", arg);
-        long millis = Long.parseLong(arg);
+        long millis = parseLong(arg);
         assert millis >= 0L : "Sleep timeout must be >= 0";
 
         for (int i = threadId; i < nextThreadId; i++) {
@@ -1220,7 +1234,7 @@ public class ConcurrentTestCommandScript
     }
 
     private void addExtraCommands(Iterable<String> commands, String state) {
-      assert state != null;
+      requireNonNull(state, "state");
 
       for (int i = 0, n = STATE_TABLE.length; i < n; i++) {
         if (state.equals(STATE_TABLE[i].state)) {
@@ -1244,13 +1258,13 @@ public class ConcurrentTestCommandScript
      * seeing the command.
      */
     private Map<String, String> lookupState(String state) {
-      assert state != null;
+      requireNonNull(state, "state");
 
       for (StateAction a : STATE_TABLE) {
         if (state.equals(a.state)) {
           StateDatum[] stateData = a.stateData;
 
-          Map<String, String> result = new HashMap<String, String>();
+          Map<String, String> result = new HashMap<>();
           for (StateDatum datum : stateData) {
             result.put(datum.x, datum.y);
           }
@@ -1385,7 +1399,7 @@ public class ConcurrentTestCommandScript
             nth = 1;
             if (tokenizer.hasMoreTokens()) {
               token = tokenizer.nextToken();
-              nth = Integer.parseInt(token);
+              nth = parseInt(token);
             }
           }
         }
@@ -1396,8 +1410,7 @@ public class ConcurrentTestCommandScript
       this.total = total;
     }
 
-    protected void doExecute(ConcurrentTestCommandExecutor executor)
-        throws SQLException {
+    protected void doExecute(ConcurrentTestCommandExecutor executor) {
       Integer threadId = executor.getThreadId();
       BufferedWriter out = threadBufferedWriters.get(threadId);
       threadResultsReaders.put(
@@ -1413,8 +1426,7 @@ public class ConcurrentTestCommandScript
       this.msg = msg;
     }
 
-    protected void doExecute(ConcurrentTestCommandExecutor executor)
-        throws SQLException {
+    protected void doExecute(ConcurrentTestCommandExecutor executor) {
       storeMessage(executor.getThreadId(), msg);
     }
   }
@@ -1424,9 +1436,7 @@ public class ConcurrentTestCommandScript
 
     private final ConcurrentTestPluginCommand pluginCommand;
 
-    private PluginCommand(
-        String command,
-        String params) throws IOException {
+    private PluginCommand(String command, String params) {
       ConcurrentTestPlugin plugin = pluginForCommand.get(command);
       pluginCommand = plugin.getCommandFor(command, params);
     }
@@ -1444,7 +1454,7 @@ public class ConcurrentTestCommandScript
               return exec.getConnection();
             }
 
-            public Statement getCurrentStatement() {
+            public @Nullable Statement getCurrentStatement() {
               return exec.getStatement();
             }
           };
@@ -1460,7 +1470,7 @@ public class ConcurrentTestCommandScript
   /** Shell command. */
   private class ShellCommand extends AbstractCommand {
     private final String command;
-    private List<String> argv;      // the command, parsed and massaged
+    private final List<String> argv;      // the command, parsed and massaged
 
     private ShellCommand(String command) {
       this.command = command;
@@ -1513,7 +1523,7 @@ public class ConcurrentTestCommandScript
   /** Command that has a timeout. */
   // TODO: replace by super.CommmandWithTimeout
   private abstract static class CommandWithTimeout extends AbstractCommand {
-    private long timeout;
+    private final long timeout;
 
     private CommandWithTimeout(long timeout) {
       this.timeout = timeout;
@@ -1536,11 +1546,7 @@ public class ConcurrentTestCommandScript
   /** Command with timeout and row limit. */
   private abstract static class CommandWithTimeoutAndRowLimit
       extends CommandWithTimeout {
-    private int rowLimit;
-
-    private CommandWithTimeoutAndRowLimit(long timeout) {
-      this(timeout, 0);
-    }
+    private final int rowLimit;
 
     private CommandWithTimeoutAndRowLimit(long timeout, int rowLimit) {
       super(timeout);
@@ -1560,7 +1566,7 @@ public class ConcurrentTestCommandScript
    * timeout and row limit.
    */
   private class SelectCommand extends CommandWithTimeoutAndRowLimit {
-    private String sql;
+    private final String sql;
 
     private SelectCommand(String sql) {
       this(sql, 0, 0);
@@ -1617,7 +1623,7 @@ public class ConcurrentTestCommandScript
    * timeout.
    */
   private class SqlCommand extends CommandWithTimeout {
-    private String sql;
+    private final String sql;
 
     private SqlCommand(String sql) {
       super(0);
@@ -1673,9 +1679,7 @@ public class ConcurrentTestCommandScript
         } else {
           int rows = stmt.getUpdateCount();
           if (rows != 1) {
-            storeMessage(
-                executor.getThreadId(),
-                String.valueOf(rows) + " rows affected.");
+            storeMessage(executor.getThreadId(), rows + " rows affected.");
           } else {
             storeMessage(
                 executor.getThreadId(),
@@ -1708,7 +1712,7 @@ public class ConcurrentTestCommandScript
    * CloseCommand closes and discards the prepared statement.
    */
   private class PrepareCommand extends AbstractCommand {
-    private String sql;
+    private final String sql;
 
     private PrepareCommand(String sql) {
       this.sql = sql;
@@ -1768,7 +1772,7 @@ public class ConcurrentTestCommandScript
     // print final summary, rows & elapsed time.
     private final boolean totaled;
 
-    private long baseTime = 0;
+    private final long baseTime;
     private int rowCount = 0;
     private int ncols = 0;
     private int[] widths;
@@ -1985,14 +1989,12 @@ public class ConcurrentTestCommandScript
     boolean debug = false;          // -g
     String server;                  // -u
     String driver;                  // -d
-    String user;                    // -n
-    String password;                // -p
-    List<String> bindings;          // VAR=VAL
-    List<String> files;             // FILE
+    @Nullable String user;                    // -n
+    @Nullable String password;                // -p
+    final List<String> bindings = new ArrayList<>(); // VAR=VAL
+    final List<String> files = new ArrayList<>(); // FILE
 
     Tool() {
-      bindings = new ArrayList<>();
-      files = new ArrayList<>();
     }
 
     // returns 0 on success, 1 on error, 2 on bad invocation.
